import { ClientOptions, Client } from 'discord.js';
import { Worker, ClusterSettings } from 'node:cluster';
import EventEmitter from 'node:events';
import { Serializable } from 'node:child_process';
import { RequestOptions } from 'node:https';
import { WebSocketShardEvents, FetchingStrategyOptions, IShardingStrategy, WebSocketManager, WebSocketShardDestroyOptions, WebSocketShardStatus, WebSocketShard, IContextFetchingStrategy, SessionInfo } from '@discordjs/ws';
import { Worker as Worker$1, ResourceLimits } from 'node:worker_threads';
import { Collection } from '@discordjs/collection';

/**
 * A wrapper for @discordjs/ws to work exclusively with Indomitable's dynamic concurrency with support for abort controller
 */
declare class ConcurrencyManager {
    private readonly throttler;
    private readonly signals;
    constructor(concurrency: number);
    /**
     * Method to try and acquire a lock for identify
     */
    waitForIdentify(shardId: number): Promise<void>;
    /**
     * Aborts an acquire lock request
     */
    abortIdentify(shardId: number): void;
}

/**
 * Hoisted Environmental Variable for ease of fetching
 */
declare const EnvProcessData: {
    clusterId: number;
    clusterCount: number;
    shardIds: number[];
    shardCount: number;
};
/**
 * Internal operation codes for the cluster -> thread
 */
declare enum MainStrategyOps {
    CONNECT = "connect",
    DESTROY = "destroy",
    SEND = "send",
    STATUS = "status",
    RECONNECT = "reconnect"
}
/**
 * Internal operation codes for the thread <- cluster
 */
declare enum ThreadStrategyOps {
    REQUEST_IDENTIFY = "requestIdentify",
    CANCEL_IDENTIFY = "cancelIdentify",
    SHARD_EVENT = "shardEvent",
    RETRIEVE_SESSION = "retrieveSession",
    UPDATE_SESSION = "updateSession"
}
/**
 * Internal operation codes
 */
declare enum InternalOps {
    EVAL = "eval",
    RESTART = "restart",
    RESTART_ALL = "restartAll",
    DESTROY_CLIENT = "destroyClient",
    REQUEST_IDENTIFY = "requestIdentify",
    CANCEL_IDENTIFY = "cancelIdentify",
    SESSION_INFO = "sessionInfo",
    PING = "ping"
}
/**
 * Events for internal use
 */
declare enum ClientEvents {
    READY = "ready",
    SHARD_READY = "shardReady",
    SHARD_RECONNECT = "shardReconnect",
    SHARD_RESUME = "shardResume",
    SHARD_DISCONNECT = "shardDisconnect",
    ERROR = "ERROR"
}
/**
 * Events emitted by Indomitable
 */
declare enum LibraryEvents {
    DEBUG = "debug",
    MESSAGE = "message",
    ERROR = "error",
    WORKER_FORK = "workerFork",
    WORKER_READY = "workerReady",
    WORKER_EXIT = "workerExit",
    SHARD_READY = "shardReady",
    SHARD_RECONNECT = "shardReconnect",
    SHARD_RESUME = "shardResume",
    SHARD_DISCONNECT = "shardDisconnect",
    CLIENT_READY = "clientReady",
    RAW = "raw"
}
/**
 * Type for raw ipc message
 */
declare enum RawIpcMessageType {
    MESSAGE = "message",
    RESPONSE = "response",
    ERROR = "error"
}
/**
 * Type for raw ipc messages of cluster -> thread
 */
interface MainStrategyData {
    op: MainStrategyOps;
    data: any;
    internal: true;
}
/**
 * Type for raw ipc messages of cluster <- thread
 */
interface ThreadStrategyData {
    op: ThreadStrategyOps;
    event: WebSocketShardEvents;
    data: any;
    shardId: number;
    internal: true;
}
/**
 * Data structure representing an internal event
 */
interface InternalOpsData {
    op: InternalOps;
    data: any;
    internal: true;
}
/**
 * Data structure representing an internal discord.js event
 */
interface ClientEventData {
    op: ClientEvents;
    data: any;
    internal: true;
}
/**
 * Data structure representing an internal error
 */
interface IpcErrorData {
    name: string;
    reason: string;
    stack: string;
}
/**
 * Data structure representing IPC data
 */
interface Transportable {
    content: any;
    repliable?: boolean;
    signal?: AbortSignal;
}
/**
 * Data structure representing an internal abort data
 */
interface InternalAbortSignal {
    listener: () => void;
    signal: AbortSignal;
}
interface SavePromiseOptions {
    id: string;
    resolve: (data: unknown) => void;
    reject: (reason: unknown) => void;
    signal?: AbortSignal | undefined;
}
/**
 * Data structure representing a generated abort controller instance
 */
interface AbortableData {
    controller: AbortController;
    timeout: NodeJS.Timeout;
}
/**
 * Internal promise data tracking
 */
interface InternalPromise {
    resolve: Function;
    reject: Function;
    controller?: InternalAbortSignal;
}
/**
 * Data structure representing internal IPC data
 */
interface RawIpcMessage {
    id: string | null;
    content: any;
    internal: true;
    type: RawIpcMessageType;
}
/**
 * Data structure representing an IPC message
 */
interface Message {
    reply: (data: any) => void;
    content: any;
    repliable: boolean;
}
/**
 * Data structure representing a Discord session
 */
interface SessionObject {
    url: string;
    shards: number;
    session_start_limit: {
        total: number;
        remaining: number;
        reset_after: number;
        max_concurrency: number;
    };
}
/**
 * Wrapper function for fetching data using HTTP
 * @param url URL of resource to fetch
 * @param options RequestOptions to modify behavior
 * @returns A promise containing data fetched, or an error
 */
declare function Fetch(url: string | URL, options: RequestOptions): Promise<any>;
/**
 * Fetch sessions from discord
 * @param token Bot token
 * @returns A promise containing a session object
 */
declare function FetchSessions(token: string): Promise<SessionObject>;
/**
 * Modify an array to contain the specified amount of chunks
 * @param original An array of data
 * @param chunks The amount of chunks to transform into
 * @returns A modified array
 */
declare function Chunk(original: any[], chunks: number): any[];
/**
 * Wait for a specific amount of time (timeout)
 * @param ms Time to wait in milliseconds
 * @returns A promise that resolves in x seconds
 */
declare function Delay(ms: number): Promise<void>;
/**
 * Creates an abortable request with controller and timeout
 * @param delay Time before an abort error throws
 * @returns An abortable data with controller and timeout
 */
declare function MakeAbortableRequest(delay: number): AbortableData;

type Util_AbortableData = AbortableData;
declare const Util_Chunk: typeof Chunk;
type Util_ClientEventData = ClientEventData;
type Util_ClientEvents = ClientEvents;
declare const Util_ClientEvents: typeof ClientEvents;
declare const Util_Delay: typeof Delay;
declare const Util_EnvProcessData: typeof EnvProcessData;
declare const Util_Fetch: typeof Fetch;
declare const Util_FetchSessions: typeof FetchSessions;
type Util_InternalAbortSignal = InternalAbortSignal;
type Util_InternalOps = InternalOps;
declare const Util_InternalOps: typeof InternalOps;
type Util_InternalOpsData = InternalOpsData;
type Util_InternalPromise = InternalPromise;
type Util_IpcErrorData = IpcErrorData;
type Util_LibraryEvents = LibraryEvents;
declare const Util_LibraryEvents: typeof LibraryEvents;
type Util_MainStrategyData = MainStrategyData;
type Util_MainStrategyOps = MainStrategyOps;
declare const Util_MainStrategyOps: typeof MainStrategyOps;
declare const Util_MakeAbortableRequest: typeof MakeAbortableRequest;
type Util_Message = Message;
type Util_RawIpcMessage = RawIpcMessage;
type Util_RawIpcMessageType = RawIpcMessageType;
declare const Util_RawIpcMessageType: typeof RawIpcMessageType;
type Util_SavePromiseOptions = SavePromiseOptions;
type Util_SessionObject = SessionObject;
type Util_ThreadStrategyData = ThreadStrategyData;
type Util_ThreadStrategyOps = ThreadStrategyOps;
declare const Util_ThreadStrategyOps: typeof ThreadStrategyOps;
type Util_Transportable = Transportable;
declare namespace Util {
  export { type Util_AbortableData as AbortableData, Util_Chunk as Chunk, type Util_ClientEventData as ClientEventData, Util_ClientEvents as ClientEvents, Util_Delay as Delay, Util_EnvProcessData as EnvProcessData, Util_Fetch as Fetch, Util_FetchSessions as FetchSessions, type Util_InternalAbortSignal as InternalAbortSignal, Util_InternalOps as InternalOps, type Util_InternalOpsData as InternalOpsData, type Util_InternalPromise as InternalPromise, type Util_IpcErrorData as IpcErrorData, Util_LibraryEvents as LibraryEvents, type Util_MainStrategyData as MainStrategyData, Util_MainStrategyOps as MainStrategyOps, Util_MakeAbortableRequest as MakeAbortableRequest, type Util_Message as Message, type Util_RawIpcMessage as RawIpcMessage, Util_RawIpcMessageType as RawIpcMessageType, type Util_SavePromiseOptions as SavePromiseOptions, type Util_SessionObject as SessionObject, type Util_ThreadStrategyData as ThreadStrategyData, Util_ThreadStrategyOps as ThreadStrategyOps, type Util_Transportable as Transportable };
}

/**
 * Base class where primary and worker ipc inherits
 */
declare abstract class BaseIpc {
    readonly manager: Indomitable | EventEmitter;
    protected readonly promises: Map<string, InternalPromise>;
    protected constructor(manager: Indomitable | EventEmitter);
    /**
     * Number of promises pending to be resolved
     */
    get pendingPromises(): number;
    /**
     * Rejects all the pending promises
     */
    flushPromises(reason: string): void;
    /**
     * Raw send method without abort controller handling
     * @param transportable Data to send
     */
    send(transportable: Transportable): Promise<unknown | undefined>;
    /**
     * Taps into message event of worker or primary process to handle ipc communication
     * @internal
     */
    handleRawResponse(data: Serializable, errorCallback: (error: unknown) => any): Promise<boolean | void>;
    protected waitForPromise(options: SavePromiseOptions): void;
    private handlePromise;
    private handleUnparsedMessage;
    protected emitMessage(message: Message): void;
    protected abstract available(): boolean;
    protected abstract sendData(data: RawIpcMessage): void;
    protected abstract handleMessage(message: Message): Promise<void>;
}

/**
 * Primary ipc class. Only initialized at main process
 */
declare class MainWorker extends BaseIpc {
    readonly cluster: ClusterManager;
    constructor(cluster: ClusterManager);
    protected available(): boolean;
    protected sendData(data: RawIpcMessage): void;
    protected handleMessage(message: Message): Promise<void>;
}

/**
 * Options for child processes
 */
interface ClusterManagerOptions {
    id: number;
    shards: number[];
    manager: Indomitable;
}
/**
 * A class to manage a cluster
 */
declare class ClusterManager {
    readonly manager: Indomitable;
    readonly id: number;
    readonly ipc: MainWorker;
    shards: number[];
    started: boolean;
    ready: boolean;
    readyAt: number;
    worker?: Worker;
    /**
     * @param options.id ClusterId of this Cluster Manager being created
     * @param options.shards An array of numbers representing the shards that this cluster controls
     * @param options.manager Indomitable instance that spawned this cluster
     */
    constructor(options: ClusterManagerOptions);
    /**
     * Destroy associated worker process
     * @param signal Process exit signal
     */
    destroy(signal?: string): void;
    /**
     * Respawn associated worker process
     * @param delay Time to wait before restarting worker process
     */
    respawn(delay?: number): Promise<void>;
    /**
     * Spawn a worker process
     */
    spawn(): Promise<void>;
    /**
     * Remove all listeners on attached worker process and free from memory
     */
    private cleanup;
    /**
     * Waits for this cluster to be ready
     * @returns A promise that resolves to void
     * @internal
     */
    private wait;
}

/**
 * Options to control Indomitable behavior
 */
interface IndomitableOptions {
    clusterCount?: number | 'auto';
    shardCount?: number | 'auto';
    clientOptions?: ClientOptions;
    clusterSettings?: ClusterSettings;
    ipcTimeout?: number;
    spawnTimeout?: number;
    spawnDelay?: number;
    autoRestart?: boolean;
    waitForReady?: boolean;
    handleConcurrency?: boolean;
    client: typeof Client;
    token: string;
}
interface ReconfigureOptions {
    clusters?: number;
    shards?: number;
}
interface ShardEventData {
    clusterId: number;
    shardId?: number;
    replayed?: number;
    event?: CloseEvent;
}
declare interface Indomitable {
    /**
     * Emitted when data useful for debugging is produced
     * @eventProperty
     */
    on(event: 'debug', listener: (message: string) => void): this;
    /**
     * Emitted when an IPC message is received
     * @eventProperty
     */
    on(event: 'message', listener: (message: Message | unknown) => void): this;
    /**
     * Emitted when an error occurs
     * @eventProperty
     */
    on(event: 'error', listener: (error: unknown) => void): this;
    /**
     * Emitted when a new worker process is forked
     * @eventProperty
     */
    on(event: 'workerFork', listener: (cluster: ClusterManager) => void): this;
    /**
     * Emitted when a worker process is ready
     * @eventProperty
     */
    on(event: 'workerReady', listener: (cluster: ClusterManager) => void): this;
    /**
     * Emitted when a worker process exits
     * @eventProperty
     */
    on(event: 'workerExit', listener: (code: number | null, signal: string | null, cluster: ClusterManager) => void): this;
    /**
     * Emitted when a Discord.js shard is ready
     * @eventProperty
     */
    on(event: 'shardReady', listener: (event: ShardEventData) => void): this;
    /**
     * Emitted when a Discord.js shard is reconnecting
     * @eventProperty
     */
    on(event: 'shardReconnect', listener: (event: ShardEventData) => void): this;
    /**
     * Emitted when a Discord.js shard resumes
     * @eventProperty
     */
    on(event: 'shardResume', listener: (event: ShardEventData) => void): this;
    /**
     * Emitted when a Discord.js shard disconnects
     * @eventProperty
     */
    on(event: 'shardDisconnect', listener: (event: ShardEventData) => void): this;
    /**
     * Emitted when a Discord.js client is ready
     * @eventProperty
     */
    on(event: 'clientReady', listener: (event: ShardEventData) => void): this;
    /**
     * Emitted on every ipc message the handler receives
     * @eventProperty
     */
    on(event: 'raw', listener: (event: unknown) => void): this;
    once(event: 'debug', listener: (message: string) => void): this;
    once(event: 'message', listener: (message: Message | unknown) => void): this;
    once(event: 'error', listener: (error: unknown) => void): this;
    once(event: 'workerFork', listener: (cluster: ClusterManager) => void): this;
    once(event: 'workerReady', listener: (cluster: ClusterManager) => void): this;
    once(event: 'workerExit', listener: (code: number | null, signal: string | null, cluster: ClusterManager) => void): this;
    once(event: 'shardReady', listener: (event: ShardEventData) => void): this;
    once(event: 'shardReconnect', listener: (event: ShardEventData) => void): this;
    once(event: 'shardResume', listener: (event: ShardEventData) => void): this;
    once(event: 'shardDisconnect', listener: (event: ShardEventData) => void): this;
    once(event: 'clientReady', listener: (event: ShardEventData) => void): this;
    once(event: 'raw', listener: (event: unknown) => void): this;
    off(event: 'debug', listener: (message: string) => void): this;
    off(event: 'message', listener: (message: Message | unknown) => void): this;
    off(event: 'error', listener: (error: unknown) => void): this;
    off(event: 'workerFork', listener: (cluster: ClusterManager) => void): this;
    off(event: 'workerReady', listener: (cluster: ClusterManager) => void): this;
    off(event: 'workerExit', listener: (code: number | null, signal: string | null, cluster: ClusterManager) => void): this;
    off(event: 'shardReady', listener: (event: ShardEventData) => void): this;
    off(event: 'shardReconnect', listener: (event: ShardEventData) => void): this;
    off(event: 'shardResume', listener: (event: ShardEventData) => void): this;
    off(event: 'shardDisconnect', listener: (event: ShardEventData) => void): this;
    off(event: 'clientReady', listener: (event: ShardEventData) => void): this;
    off(event: 'raw', listener: (event: unknown) => void): this;
}
/**
 * The main Indomitable class, exposing all functionality.
 */
declare class Indomitable extends EventEmitter {
    clusterCount: number | 'auto';
    shardCount: number | 'auto';
    cachedSession?: SessionObject;
    concurrencyManager?: ConcurrencyManager;
    readonly clientOptions: ClientOptions;
    readonly clusterSettings: ClusterSettings;
    readonly ipcTimeout: number;
    readonly spawnTimeout: number;
    readonly spawnDelay: number;
    readonly autoRestart: boolean;
    readonly waitForReady: boolean;
    readonly handleConcurrency: boolean;
    readonly client: typeof Client;
    readonly clusters: Map<number, ClusterManager>;
    private readonly spawnQueue;
    private readonly token;
    private busy;
    /**
     * @param [options.clusterCount=auto] The amount of clusters to spawn. Expects a number or 'auto'
     * @param [options.shardCount=auto] The number of shards to create. Expects a number or 'auto'
     * @param [options.clientOptions] Options for the Discord.js client
     * @param [options.clusterSettings] Options for the forked process
     * @param [options.ipcTimeout] Time to wait before an ipc request aborts
     * @param [options.spawnTimeout] Time to wait before reporting a failed child process spawn
     * @param [options.spawnDelay] Time to wait before spawning another child process
     * @param [options.autoRestart] Whether to automatically restart shards that have been killed unintentionally
     * @param [options.waitForReady] Whether to wait for clusters to be ready before spawning a new one
     * @param [options.handleConcurrency] Whether you want to handle concurrency properly. Enabling this may result into more stable connection
     * @param [options.client] A Discord.js client class or a modified Discord.js client class
     * @param options.token Discord bot token
     */
    constructor(options: IndomitableOptions);
    /**
     * Checks the internal private flag if Indomitable is busy
     * @returns Number of clusters in queue
     */
    get isBusy(): boolean;
    /**
     * Gets how many clusters are waiting to be spawned
     * @returns Number of clusters in queue
     */
    get inSpawnQueueCount(): number;
    /**
     * Gets the current session info of the bot token Indomitable currently handles
     * @returns Session Info
     */
    fetchSessions(force?: boolean): Promise<SessionObject>;
    /**
     * Spawn a new ShardClient if this instance is a child process, or start a new cluster and IPC server if this instance is the primary process
     * @returns A promise that resolves to void
     */
    spawn(): Promise<void>;
    /**
     * Restart specified cluster if this instance is the primary process
     * @param clusterId Id of cluster to restart
     * @returns A promise that resolves to void
     */
    restart(clusterId: number): Promise<void>;
    /**
     * Restart all clusters if this instance is the primary process
     * @returns A promise that resolves to void
     */
    restartAll(): Promise<void>;
    /**
     * Sends a message to a specific cluster
     * @returns A promise that resolves to undefined or an unknown value depending on how you reply to it
     */
    send(id: number, transportable: Transportable): Promise<unknown | undefined>;
    /**
     * Sends a message on all clusters
     * @returns An array of promise that resolves to undefined or an unknown value depending on how you reply to it
     */
    broadcast(transportable: Transportable): Promise<unknown[] | undefined>;
    /**
     * Reconfigures to launch more shards / clusters without killing the existing processes if possible to avoid big downtimes
     * @remarks Never execute restart() or restartAll() during this process or else you will double restart that cluster / all clusters
     * @returns A promise that resolves to void
     */
    reconfigure(options?: ReconfigureOptions): Promise<void>;
    /**
     * Adds a cluster to spawn queue
     * @internal
     */
    addToSpawnQueue(...clusters: ClusterManager[]): Promise<void>;
    /**
     * Destroys the client on a cluster
     * @internal
     */
    private destroyClusterClient;
    /**
     * Processes the cluster queue
     * @internal
     */
    private processQueue;
}

interface PartialInternalEvents {
    op: ClientEvents;
    data: {
        clusterId: number;
        shardId?: number;
        replayed?: number;
        event?: CloseEvent;
        ipcId?: string;
    };
}
declare class ShardClient {
    readonly manager: Indomitable;
    readonly client: Client;
    readonly clusterId: number;
    constructor(manager: Indomitable);
    start(token: string): Promise<void>;
    private send;
}

/**
 * Basic worker ipc class, basic child process ipc handler
 */
declare class BaseWorker extends BaseIpc {
    constructor(manager?: Indomitable | EventEmitter);
    protected available(): boolean;
    protected sendData(data: RawIpcMessage): void;
    protected handleMessage(message: Message): Promise<void>;
}

/**
 * Extended worker ipc class, shard client util ipc class
 */
declare class ClientWorker extends BaseWorker {
    readonly shard: ShardClientUtil;
    constructor(shard: ShardClientUtil, manager: Indomitable);
    protected emitMessage(message: Message): void;
    protected handleMessage(message: Message): Promise<void>;
}

declare interface ShardClientUtil {
    /**
     * Emitted when an IPC message from parent process is received
     * @eventProperty
     */
    on(event: 'message', listener: (message: Message | unknown) => void): this;
    once(event: 'message', listener: (message: Message | unknown) => void): this;
    off(event: 'message', listener: (message: Message | unknown) => void): this;
}
/**
 * A class that replaces d.js stock shard client util. The class is built similar to it with minor changes
 */
declare class ShardClientUtil extends EventEmitter {
    client: Client;
    readonly ipc: ClientWorker;
    readonly clusterId: number;
    readonly clusterCount: number;
    readonly shardIds: number[];
    readonly shardCount: number;
    constructor(client: Client, manager: Indomitable);
    /**
     * Gets the current ipc delay
     * @returns A promise that resolves to delay in nanoseconds
     */
    ping(): Promise<number>;
    /**
     * Evaluates a script or function on all clusters in the context of the client
     * @returns A promise that resolves to an array of code results
     */
    broadcastEval(script: Function, context?: any): Promise<unknown[]>;
    /**
     * Fetches a client property value on all clusters
     * @returns A promise that resolves to an array of code results
     */
    fetchClientValues(prop: string): Promise<unknown[]>;
    /**
     * Gets the cached session info or fetches an updated session info
     * @param update If you want to fetch and update the cached session info
     * @returns A session object
     */
    fetchSessions(update?: boolean): Promise<SessionObject>;
    /**
     * Restarts the given cluster from the clusterId given
     * @returns A promise that resolves to void
     */
    restart(clusterId: number): Promise<undefined>;
    /**
     * Restarts all the clusters Indomitable handles sequentially
     * @returns A promise that resolves to void
     */
    restartAll(): Promise<undefined>;
    /**
     * Sends a message to primary process
     * @returns A promise that resolves to void or a repliable object
     */
    send(transportable: Transportable): Promise<unknown | undefined>;
}

interface WorkerData extends FetchingStrategyOptions {
    shardId: number;
}
interface IndomitableWorker {
    thread: Worker$1;
    ipc: MainStrategyWorker;
}
interface IndomitableStrategyOptions {
    path?: string;
    resourceLimits?: ResourceLimits;
}
declare class IndomitableStrategy implements IShardingStrategy {
    readonly manager: WebSocketManager;
    readonly ipc: BaseWorker;
    readonly workers: Collection<number, IndomitableWorker>;
    private readonly options;
    constructor(manager: WebSocketManager, ipc: BaseWorker, options?: IndomitableStrategyOptions);
    spawn(shardIds: number[]): Promise<void>;
    connect(): Promise<void>;
    destroy(data?: Omit<WebSocketShardDestroyOptions, 'recover'>): Promise<void>;
    reconnect(shardId: number): Promise<void>;
    send(shardId: number, data: any): Promise<void>;
    fetchStatus(): Promise<Collection<number, WebSocketShardStatus>>;
    private createWorker;
}

declare class MainStrategyWorker extends BaseIpc {
    readonly id: number;
    readonly thread: Worker$1;
    readonly strategy: IndomitableStrategy;
    constructor(id: number, thread: Worker$1, strategy: IndomitableStrategy);
    protected available(): boolean;
    protected sendData(data: RawIpcMessage): void;
    protected handleMessage(message: Message): Promise<void>;
}

declare class ThreadStrategyWorker extends BaseIpc {
    private shard;
    constructor();
    build(shard: WebSocketShard): void;
    protected available(): boolean;
    protected sendData(data: RawIpcMessage): void;
    protected handleMessage(message: Message): Promise<void>;
}

declare class IndomitableFetchingStrategy implements IContextFetchingStrategy {
    private readonly ipc;
    readonly options: FetchingStrategyOptions;
    constructor(ipc: ThreadStrategyWorker, options: FetchingStrategyOptions);
    retrieveSessionInfo(shardId: number): Promise<SessionInfo | null>;
    updateSessionInfo(shardId: number, sessionInfo: SessionInfo | null): Promise<void>;
    waitForIdentify(shardId: number, signal: AbortSignal): Promise<void>;
    private abortIdentify;
}

export { BaseIpc, BaseWorker, ClientWorker, ClusterManager, type ClusterManagerOptions, ConcurrencyManager, Indomitable, IndomitableFetchingStrategy, type IndomitableOptions, IndomitableStrategy, type IndomitableStrategyOptions, type IndomitableWorker, MainStrategyWorker, MainWorker, type PartialInternalEvents, type ReconfigureOptions, ShardClient, ShardClientUtil, type ShardEventData, ThreadStrategyWorker, Util as Utils, type WorkerData };
